//
//  TRMovieInspector(Attributes).m
//  Tryst
//
//  Created by Francisco Tolmasky on Sat Dec 06 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "NSString+TRDrawingAdditions.h"
#import "TRMovieLabelCell.h"
#import "TRMovieInspector.h"


@implementation TRMovieInspector (Attributes)

// ======================================================================
// labelAttributesForProperty:
// - Returns an NSDictionary of label attributes and aProperty as the 
//   value for kPropertyAttributeName.
// ======================================================================

- (NSDictionary *)labelAttributesForProperty:(NSString *)aProperty
{
    NSFont *font= [NSFont boldSystemFontOfSize: 11];
    NSDictionary *labelAttributes= [[NSDictionary alloc] initWithObjectsAndKeys:
        font, NSFontAttributeName,
        [NSColor grayColor], NSForegroundColorAttributeName,
        [NSCursor arrowCursor], NSCursorAttributeName,
        @"the tool tip", NSToolTipAttributeName,
        aProperty, kLabelAttributeName, nil];

    return labelAttributes;
}

// ======================================================================
// placeholderAttributesForIdentifier: placeholderString: font:
// - Returns an NSDictionary with placeholder attributes with font aFont
//   and placeholder string aPlaceholderString.
// ======================================================================

- (NSDictionary *)placeholderAttributesForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    placeholderString:(NSString *)aPlaceholderString
    font:(NSFont *)aFont
{
    NSDictionary *placeholderAttributes= [[NSDictionary alloc] initWithObjectsAndKeys:
        aFont, NSFontAttributeName,
        [NSColor grayColor], NSForegroundColorAttributeName,
        aProperty, kPropertyAttributeName,
        anIdentifier, kIdentifierAttributeName,
        aPlaceholderString, kPlaceholderAttributeName, nil];
    return placeholderAttributes;
}

// ======================================================================
// typingAttributesForIdentifier: placeholderString: font:
// - Returns an NSDictionary with typing attributes with font aFont,
//   identifier anIdentifier, and placeholder string aPlaceholderString.
// ======================================================================

- (NSDictionary *)typingAttributesForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    placeholderString:(NSString *)aPlaceholderString
    font:(NSFont *)aFont
{
    NSDictionary *typingAttributes= [[NSDictionary alloc] initWithObjectsAndKeys:
        aFont, NSFontAttributeName,
        [NSColor blackColor], NSForegroundColorAttributeName,
        aProperty, kPropertyAttributeName,
        anIdentifier, kIdentifierAttributeName,
        aPlaceholderString, kPlaceholderAttributeName, nil];
    return typingAttributes;
}

// ======================================================================
// typingAttributes
// - Returns an NSDictionary with the surrounding typing attributes.
// ======================================================================

- (NSDictionary *)typingAttributes
{
    NSString    *propertyForProposal= nil,
                *identifierForProposal= nil,
                *placeholderStringForProposal= nil;
    
    NSFont *fontForProposal;
    
    if ([self isEditing])
    {
        NSTextStorage *textStorage= [self textStorage];
        
        propertyForProposal= [textStorage attribute: kPropertyAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
        identifierForProposal= [textStorage attribute: kIdentifierAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
        placeholderStringForProposal= [textStorage attribute: kPlaceholderAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
        fontForProposal= [textStorage attribute: NSFontAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
    }
    return (placeholderStringForProposal?
        [self typingAttributesForProperty: propertyForProposal identifier: identifierForProposal placeholderString: placeholderStringForProposal font: fontForProposal]:nil);
}

// ======================================================================
// - tabStopAttributesForScalarProperties
// * Returns an NSDictionary with tab stop attributes to be used with 
//   scalar properties (such as kTRMovieTitleProperty). Line break mode  
//   set to NSLineBreakByWordWrapping.
// ======================================================================

- (NSDictionary *)tabStopAttributesForScalarProperties
{
    static NSDictionary *TRMovieInspectorTabStopAttributesForScalarProperties= nil;
    if (!TRMovieInspectorTabStopAttributesForScalarProperties)
    {
        NSMutableParagraphStyle *paragraphStyle= [[NSMutableParagraphStyle alloc] init];
        NSTextTab *rightTabStop= [[NSTextTab alloc] initWithType: NSRightTabStopType location: kLeftTabLocation];
        NSTextTab *leftTabStop= [[NSTextTab alloc] initWithType: NSLeftTabStopType location: kRightTabLocation];
        [paragraphStyle setTabStops: [NSArray arrayWithObjects: rightTabStop, leftTabStop, nil]];
        [paragraphStyle setHeadIndent: kRightTabLocation];
        [paragraphStyle setLineBreakMode: NSLineBreakByWordWrapping];

        TRMovieInspectorTabStopAttributesForScalarProperties=
            [[NSDictionary alloc] initWithObjectsAndKeys: paragraphStyle, NSParagraphStyleAttributeName, nil];
    }
    return TRMovieInspectorTabStopAttributesForScalarProperties;
}

// ======================================================================
// - tabStopAttributesForMultiValueProperties
// * Returns an NSDictionary with tab stop attributes to be used with 
//   mutlivalue properties. Line break mode set to 
//   NSLineBreakByCharWrapping.
// ======================================================================

- (NSDictionary *)tabStopAttributesForMultiValueProperties
{
    static NSDictionary *TRMovieInspectorTabStopAttributesForMultiValueProperties= nil;
    if (!TRMovieInspectorTabStopAttributesForMultiValueProperties)
    {
        NSMutableParagraphStyle *paragraphStyle= [[NSMutableParagraphStyle alloc] init];
        NSTextTab *rightTabStop= [[NSTextTab alloc] initWithType: NSRightTabStopType location: kLeftTabLocation];
        NSTextTab *leftTabStop= [[NSTextTab alloc] initWithType: NSLeftTabStopType location: kRightTabLocation];
        [paragraphStyle setTabStops: [NSArray arrayWithObjects: rightTabStop, leftTabStop, nil]];
        [paragraphStyle setHeadIndent: kRightTabLocation];
        [paragraphStyle setLineBreakMode: NSLineBreakByCharWrapping];

        TRMovieInspectorTabStopAttributesForMultiValueProperties=
            [[NSDictionary alloc] initWithObjectsAndKeys: paragraphStyle, NSParagraphStyleAttributeName, nil];
    }
    return TRMovieInspectorTabStopAttributesForMultiValueProperties;
}

// --- Utilities Setup the UI ---

- (NSAttributedString *)fieldForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    selectedLabel:(NSString *)aSelectedLabel
    currentString:(NSString *)aCurrentString
{
    NSAttributedString *attribLabel;
    if([self isEditable])
    {
        TRMovieLabelCell *inlineMenuCell=  [[TRMovieLabelCell alloc] initWithProperty: aProperty movieInspector: self];
        
        if(aSelectedLabel)
        {
            if([inlineMenuCell indexOfItemWithTitle: aSelectedLabel]==-1) [inlineMenuCell addCustomLabel: aSelectedLabel select: YES];
            else [inlineMenuCell selectItemWithTitle: aSelectedLabel];
        }
        
        NSTextAttachment *controlAttachment= [[NSTextAttachment alloc] init];
        [controlAttachment setAttachmentCell: inlineMenuCell];
        [inlineMenuCell setAttachment: controlAttachment];
        attribLabel= [[NSAttributedString attributedStringWithAttachment: controlAttachment] mutableCopy];
        [(NSMutableAttributedString *)attribLabel addAttribute: kLabelAttributeName value: aProperty range: NSMakeRange(0,1)];
        [(NSMutableAttributedString *)attribLabel addAttribute: NSCursorAttributeName value: [NSCursor arrowCursor] range: NSMakeRange(0,1)];
    }
    else attribLabel= [aSelectedLabel truncatedStringWithAttributes: [self labelAttributesForProperty: aProperty] width: kLeftTabLocation];
    //[[NSAttributedString alloc] initWithString: aSelectedLabel attributes: [self labelAttributesForProperty: aProperty]];
    NSAttributedString *attribValue= [[NSAttributedString alloc] initWithString: aCurrentString?aCurrentString:@"Names" attributes: 
        (aCurrentString?[self typingAttributesForProperty: aProperty identifier: anIdentifier placeholderString: @"Names" font: [NSFont systemFontOfSize: 11]]:
        [self placeholderAttributesForProperty: aProperty identifier: anIdentifier placeholderString: @"Names" font: [NSFont systemFontOfSize: 11]])];

    NSMutableAttributedString *field= [[NSMutableAttributedString alloc] initWithString: @"\t"];
    [field appendAttributedString: attribLabel];
    [field appendAttributedString: [[[NSAttributedString alloc] initWithString: @"\t"] autorelease]];
    [field appendAttributedString: attribValue];
    [field appendAttributedString: [[[NSAttributedString alloc] initWithString: @"\n"] autorelease]];
    
    [attribLabel release];
    [attribValue release];
    
    return [field autorelease];
}

@end