//
//  TRMovieInspector(Editing).m
//  Tryst
//
//  Created by Francisco Tolmasky on Sat Feb 21 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMovieInspector.h"

@interface TRMovieInspector (SelectionFriend)
- (NSRange)validateProposedRange:(NSRange)proposedSelectedCharRange updateEditingInfo:(BOOL)updateEditingInfo dragging:(BOOL)isDragging;
- (void)didChangeTextAtLocation:(unsigned)aLocation;
@end

@implementation TRMovieInspector (Editing)

- (void)setCurrentEditedStartLocation:(unsigned)aLocation
{
    currentEditedStartLocation= aLocation;
}

- (BOOL)isEditing
{
    return currentEditedStartLocation!=NSNotFound;
}

- (void)endEditing
{
    // A utility routing used by the rest of our selection management code to end editing.  (see insertNewline:, etc...)
    [self setCurrentEditedStartLocation: NSNotFound];
    [self setSelectedRange: NSMakeRange(0,0)];
}

- (NSRange)currentEditingGlyphRange
{
    NSRange glyphRange= kNotFoundRange;
    if ([self isEditing])
    {
        NSTextContainer *textContainer= [self textContainer];
        NSLayoutManager *layoutManager= [textContainer layoutManager];
        NSTextStorage *textStorage= [self textStorage];
        
        // Locate the currently edited character range.
        NSRange effectiveRange;
        NSString *property= [textStorage attribute: kPropertyAttributeName atIndex: currentEditedStartLocation effectiveRange: &effectiveRange];
        if (property)
            // Convert from character range to glyph range, and glyph range/textContainer to a location in the view.
            glyphRange= [layoutManager glyphRangeForCharacterRange: effectiveRange actualCharacterRange: NULL];
    }
    return glyphRange;
}

#pragma mark Insertion

// Don't allow smart insert / delete.  That will try to add/remove spaces when copying and pasting, but we don't want that.
- (BOOL)smartInsertDeleteEnabled
{
    return NO;
}

// Don't draw the insertion point cursor unless we are editing.
- (BOOL)shouldDrawInsertionPoint
{
    return ([self isEditing]);
}

// When the user hits return, end editing.
- (void)insertNewline:(id)sender
{
    unsigned index= currentEditedStartLocation;
    [self endEditing];
    [self setSelectedRange: [self nextEditableRange: index forward: YES]];
}

- (void)insertNewlineIgnoringFieldEditor:(id)sender;
{
    [self insertNewline: self];
}

// When the user hits tab/backtab move to the next/previous editable range and start editing there.
- (void)insertTab:(id)sender
{
    unsigned index= currentEditedStartLocation;
    [self endEditing];
    [self setSelectedRange: [self nextEditableRange: index forward: YES]];
}

- (void)insertTabIgnoringFieldEditor:(id)sender
{
    [self replaceCharactersInRange: [self selectedRange] withString: @" "];
}

- (void)insertBacktab:(id)sender
{
    unsigned index= currentEditedStartLocation;
    [self endEditing];
    [self setSelectedRange: [self nextEditableRange: index forward: NO]];
}

- (void)moveDown:(id)sender
{
    if([self isEditing])
    {
        NSRange effectiveRange;
        
        [[self textStorage] attribute: kPropertyAttributeName atIndex: currentEditedStartLocation effectiveRange: &effectiveRange];
        [self setSelectedRange: NSMakeRange(NSMaxRange(effectiveRange),0)];
    }
}

- (void)moveUp:(id)sender
{
    if([self isEditing]) [self setSelectedRange: NSMakeRange(currentEditedStartLocation,0)];
}

#pragma mark Dragging Support

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
	if(!model) return NSDragOperationNone;
	
    NSLayoutManager *layoutManager= [self layoutManager];
    NSPoint localPoint= [self convertPoint: [sender draggingLocation] fromView: nil];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: localPoint inTextContainer: [self textContainer]];
    unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
    
    if(![[self textStorage] attribute: kPropertyAttributeName atIndex: characterIndex effectiveRange: NULL])
        return NSDragOperationNone;
    
    return [super draggingEntered: sender];
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	if(!model) return NSDragOperationNone;

    NSLayoutManager *layoutManager= [self layoutManager];
    NSPoint localPoint= [self convertPoint: [sender draggingLocation] fromView: nil];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: localPoint inTextContainer: [self textContainer]];
    unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
    
    if(![[self textStorage] attribute: kPropertyAttributeName atIndex: characterIndex effectiveRange: NULL])
        return NSDragOperationNone;
    
    return [super draggingUpdated: sender];
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    NSTextStorage *textStorage= [self textStorage];
    NSLayoutManager *layoutManager= [self layoutManager];
    NSPoint localPoint= [self convertPoint: [sender draggingLocation] fromView: nil];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: localPoint inTextContainer: [self textContainer]];
    NSRange proposedRange= NSMakeRange([layoutManager characterIndexForGlyphAtIndex: glyphIndex],0),
            userRange= [self rangeForUserTextChange];
    
    // Prepare string
    
    NSString    *property= [textStorage attribute: kPropertyAttributeName atIndex: proposedRange.location effectiveRange: NULL],
                *identifier= [textStorage attribute: kIdentifierAttributeName atIndex: proposedRange.location effectiveRange: NULL],
                *placeholderString= [textStorage attribute: kPlaceholderAttributeName atIndex: proposedRange.location effectiveRange: NULL];
    
    NSFont *font= [textStorage attribute: NSFontAttributeName atIndex: proposedRange.location effectiveRange: NULL];
    
    NSAttributedString *dragString= [[NSAttributedString alloc]
        initWithString: [[sender draggingPasteboard] stringForType: NSStringPboardType]
        attributes: [self typingAttributesForProperty: property identifier: identifier placeholderString: placeholderString font: font]];
    
    // Complete drag by removing string from old position and inserting in new position.

    //NSRange validatedRange= [self validateProposedRange: proposedRange updateEditingInfo: NO dragging: YES];
    //if(proposedRange.location<userRange.location)
    //    proposedRange.location-= userRange.length;//userRange.location-validatedRange.length+userRange.length;
    
    [[self textStorage]
        replaceCharactersInRange: proposedRange= [self validateProposedRange: proposedRange updateEditingInfo: NO dragging: YES]
        withAttributedString: dragString];
    
    [self didChangeTextAtLocation: proposedRange.location];

    //To account for changes
    if(userRange.location>proposedRange.location)
    {
        userRange.location-= proposedRange.length-[dragString length];
        [self setSelectedRange: userRange];
    }
    
    if([sender draggingSource]==self && [self shouldChangeTextInRange: userRange/*[self rangeForUserTextChange]*/ replacementString: @""])
        [[self textStorage] replaceCharactersInRange: userRange/*[self rangeForUserTextChange]*/ withString: @""];
    
    // Update Model
    property= [textStorage attribute: kPropertyAttributeName atIndex: proposedRange.location effectiveRange: &proposedRange];
    
    if(property==kTRMovieTitleProperty || property==kTRMovieYearProperty || property==kTRMovieGenreProperty)
        [model setValue: [[textStorage string] substringWithRange: proposedRange] forProperty: property];
    
    [dragString release];
    
    [self didChangeText];
}

@end
